(*

    ADW-Modula-2 example WindowSplitter

    Copyright: R.W.G. Hnen (rhunen@xs4all.nl)
    Based on original code by Rob Pitt.

    ported to Modula-2 by Stony Brook Software.

    2014. Edited by Frank Schoonjans

*)

MODULE WindowSplitter;
(*
 * Module   : main.c
 * Copyright: R.W.G. Hnen (rhunen@xs4all.nl)
 *
 * Based on original code by Rob Pitt.
 *
 * This file is part of the Splitter Control.
 *
 * The Splitter Control is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * The Splitter Control is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * the Splitter Control; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Log: $
 *)

FROM SYSTEM IMPORT
    CAST, ADR;

FROM SplitterControl IMPORT *;
FROM WIN32 IMPORT *;
FROM WINUSER IMPORT * EXCEPT HDWP, SLE_ERROR, SLE_MINORERROR, SLE_WARNING, SetLastErrorEx, SetDebugErrorLevel;
FROM WINX IMPORT * EXCEPT NULL;

FROM MemUtils IMPORT
    FillMemBYTE;

<*/RESOURCE:WindowSplitter*>

<*/NOWARN:F*>

CONST
    ID_SPLITTER     =1000;

    IDR_MAINMENU                    =101;

    IDM_FILE_EXIT                   =40001;
    IDM_SPLIT_HORZ                  =40002;
    IDM_SPLIT_VERT                  =40003;
    IDM_SPLIT_WIDE                  =40004;
    IDM_HELP_ABOUT                  =40005;
    IDM_SPLIT_CONSTRAIN             =40006;
    IDM_SPLIT_SHOWPOS               =40007;
    IDM_SPLIT_SHOWWIDTH             =40008;
    IDM_SPLIT_SWAPVIEWS             =40009;

NVIEWS      = 5;

viewTitles : ARRAY [0..NVIEWS-1] OF ARRAY [0..7] OF CHAR = {
    "View 0",
    "View 1",
    "View 2",
    "View 3",
    "View 4"
};

VAR
    hViewWnd : ARRAY [0..NVIEWS-1] OF HWND;
    hSplitter : HWND;

PROCEDURE MainWindowProc (
        hWnd : HWND;
        uMsg : UINT;
        wParam : WPARAM;
        lParam : LPARAM) : LRESULT [EXPORT, OSCall];
VAR
    spn         : PNMCCSPLIT;
    spx         : CCSPLIT;
    spy         : CCSPLIT;
    str         : ARRAY [0..99] OF CHAR;
    hMenu       : HMENU;
    rc          : RECT;
    ch          : INTEGER;
    flag        : DWORD;
BEGIN
    hMenu := GetMenu(hWnd);

    spx.cbSize := SIZE(spx);
    spy.cbSize := SIZE(spy);

    CASE uMsg OF
    WM_COMMAND:
        CASE LOWORD(wParam) OF
        IDM_FILE_EXIT:
            SendMessage(hWnd, WM_CLOSE, 0, 0);
        |
        IDM_SPLIT_HORZ:
            GetClientRect (hSplitter, rc);
            ch := GetMenuState (hMenu, IDM_SPLIT_HORZ, 0) BAND MF_CHECKED;

            spx.flags  := CCSP_POS;
            IF ch <> 0 THEN
                spx.pos    := 0;
            ELSE
                spx.pos := rc.right/2;
            END;
            CCsplitter_SetSplit (hSplitter, ADR(spx), NIL);
        |
        IDM_SPLIT_VERT:
            GetClientRect (hSplitter, rc);
            ch := GetMenuState (hMenu, IDM_SPLIT_VERT, 0) BAND MF_CHECKED;

            spy.flags  := CCSP_POS;
            IF ch <> 0 THEN
                spy.pos    := 0;
            ELSE
                spy.pos := rc.bottom/2;
            END;
            CCsplitter_SetSplit (hSplitter, NIL, ADR(spy));
        |
        IDM_SPLIT_WIDE:
            ch := GetMenuState (hMenu, IDM_SPLIT_WIDE, 0) BAND MF_CHECKED;
            flag := MF_UNCHECKED;
            IF ch = 0 THEN
                flag := MF_CHECKED;
            END;
            CheckMenuItem (hMenu, IDM_SPLIT_WIDE, flag);

            spx.flags  := CCSP_WIDTH;
            spy.flags  := CCSP_WIDTH;
            spx.width := 0;(*default*)
            spy.width := 0;
            IF ch = 0 THEN
                spx.width := 15;
                spy.width := 15;
            END;
            CCsplitter_SetSplit (hSplitter, ADR(spx), ADR(spy));
        |
        IDM_SPLIT_CONSTRAIN:
            ch := GetMenuState (hMenu, IDM_SPLIT_CONSTRAIN, 0) BAND MF_CHECKED;
            flag := MF_UNCHECKED;
            IF ch = 0 THEN
                flag := MF_CHECKED;
            END;
            CheckMenuItem (hMenu, IDM_SPLIT_CONSTRAIN, flag);

            spx.flags  := CCSP_FLOWER BOR CCSP_FUPPER;
            spy.flags := CCSP_FLOWER BOR CCSP_FUPPER;
            IF ch <> 0 THEN
                spx.lower  := 0;
            ELSE
                spx.lower := 25(*%*);
                spx.upper := 75(*%*);
            END;
            IF ch <> 0 THEN
                spy.upper  := 0;
            ELSE
                spy.lower := 25(*%*);
                spy.upper := 75(*%*);
            END;
            CCsplitter_SetSplit (hSplitter, ADR(spx), ADR(spy));
        |
        IDM_SPLIT_SHOWPOS:
            CCsplitter_GetSplit (hSplitter, ADR(spx), ADR(spy));
            wsprintf (str, "Splitter Position: x=%d y=%d", spx.pos, spy.pos);
            MessageBox(hWnd, str, "Info", MB_ICONINFORMATION BOR MB_OK);
        |
        IDM_SPLIT_SHOWWIDTH:
            CCsplitter_GetSplit (hSplitter, ADR(spx), ADR(spy));
            wsprintf (str, "Splitter Width: x=%d y=%d", spx.width, spy.width);
            MessageBox (hWnd, str, "Info", MB_ICONINFORMATION BOR MB_OK);
        |
        IDM_SPLIT_SWAPVIEWS:
            IF CCsplitter_GetHandle (hSplitter, 0) = hViewWnd[0] THEN
                CCsplitter_SetHandle (hSplitter, 0, hViewWnd[4]);
            ELSE
                CCsplitter_SetHandle (hSplitter, 0, hViewWnd[0]);
            END;
        |
        IDM_HELP_ABOUT:
            MessageBox (hWnd, "Non-MFC Splitter Control Example\n\nWritten by Roger Hnen\n\nOriginal code by Rob Pitt", "About Splitter", MB_ICONINFORMATION BOR MB_OK);
        ELSE
        END;
        RETURN 1;
    |
    WM_CREATE:
        CreateChildWindows (hWnd);

    |
    WM_DESTROY:
        PostQuitMessage (0);
        RETURN 1;
    |
    WM_NOTIFY:
        IF CAST(LPNMHDR, lParam)^.idFrom = ID_SPLITTER THEN
            spn := CAST(PNMCCSPLIT, lParam);
            IF spn^.hdr.code = CCSPN_SPLITCHANGED THEN
                flag := MF_CHECKED;
                IF spn^.x = 0 THEN
                    flag := MF_UNCHECKED;
                END;
                CheckMenuItem (GetMenu (hWnd), IDM_SPLIT_HORZ, flag);
                flag := MF_CHECKED;
                IF spn^.y = 0 THEN
                    flag := MF_UNCHECKED;
                END;
                CheckMenuItem (GetMenu (hWnd), IDM_SPLIT_VERT, flag);
            END;
        END;
    |
    WM_SIZE:
        MoveWindow (hSplitter, 0, 0, LOWORD (lParam), HIWORD (lParam), TRUE);
    ELSE
    END;

    RETURN DefWindowProc (hWnd, uMsg, wParam, lParam);
END MainWindowProc;

PROCEDURE CreateChildWindows (hParent : HWND);
VAR
    i   : INTEGER;
BEGIN
    hSplitter := CreateWindowEx (
        0,
        WC_CCSPLITTER,
        NIL_STR,
        WS_CHILD BOR WS_VISIBLE,
        0, 0, 0, 0,
        hParent,
        CAST(HMENU, ID_SPLITTER),
        Instance,
        NIL);

    IF hSplitter = NIL THEN
        MessageBox(NIL, "CreateWindowEx failed in CreateChildWindows", NIL_STR, MB_ICONSTOP BOR MB_OK);
        HALT(1);
    END;

    FOR i := 0 TO NVIEWS-1 DO
        hViewWnd[i] := CreateWindowEx (
            WS_EX_CLIENTEDGE,
            "EDIT",
            viewTitles[i],
            WS_CHILD BOR WS_VISIBLE,
            0, 0, 0, 0,
            hParent,
            NIL,
            Instance,
            NIL);

        IF hViewWnd[i]= NIL THEN
            MessageBox (NIL, "CreateWindowEx failed in CreateChildWindows", NIL_STR, MB_ICONSTOP BOR MB_OK);
            HALT(1);
        END;

        CCsplitter_SetHandle(hSplitter, i, hViewWnd[i]);
    END;
END CreateChildWindows;

VAR
    wc          : WNDCLASSEX;
    hWnd        : HWND;
    msg         : MSG;
BEGIN

    FillMemBYTE(ADR(wc), SIZE(wc), 0);
    wc.cbSize        := SIZE(wc);
    wc.hInstance     := Instance;
    wc.hCursor       := LoadCursor (NIL, IDC_ARROW^);
    wc.hIcon         := LoadIcon (NIL, IDI_APPLICATION^);
    wc.hIconSm       := LoadIcon (NIL, IDI_APPLICATION^);
    wc.hbrBackground := NIL;
    wc.lpfnWndProc   := MainWindowProc;
    wc.lpszMenuName  := MAKEINTRESOURCE(IDR_MAINMENU);
    wc.lpszClassName := ADR("SplitterTestMainWind");
    RegisterClassEx(wc);

    hWnd := CreateWindowEx (
        0,
        wc.lpszClassName^,
        "Splitter Test",
        WS_OVERLAPPEDWINDOW BOR WS_CLIPCHILDREN,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        320,
        280,
        NIL,
        NIL,
        Instance,
        NIL);

    IF (hWnd = NIL) THEN
        MessageBox(NIL, "CreateWindowEx failed in WinMain", NIL_STR, MB_ICONSTOP BOR MB_OK);
        HALT(1);
    END;

    ShowWindow(hWnd, SW_SHOW);

    WHILE (GetMessage (msg, NIL, 0, 0)) DO
        TranslateMessage(msg);
        DispatchMessage(msg);
    END;
END WindowSplitter.
